{$I-}
PROGRAM Text2Html;

USES Html,StrUtil;

CONST MAXLINES = 15000;

CONST TextFooter : STRING = '';
      IndexFooter : STRING = '';

      HTMLEXT = '.html';

TYPE PHref = ^THref;
     THref = RECORD
        Name:STRING[31];
        Href:STRING[127];
        Next:PHref;
     END;

VAR Index:TEXT;                 { index file }
    iNest:INTEGER;              { current nesting level in index }
    Path:STRING;                { path specified on command line }
    OutInd,OutAInd:INTEGER;     { running number of files / anchors }
    MaxOutInd:INTEGER;          { total # of files }
    OutHRef:STRING[31];         { current file }
    OutAnchor:STRING[31];       { current anchor }
    Pass:INTEGER;               { pass (1, 2) }
    HRefs:PHRef;                { known references }

    Lines:ARRAY[1..MAXLINES] OF CHAR;  { '=', '-': underlines
                                         #0:       blank line
                                         ' ':      preformatted
                                         '':      starts with bullet
                                         'b':      continuation of bulleted list
                                         'B':      continuation of bulleted list, preformatted
                                         'A':      normal text
                                         'c':      center (heading)
                                         '1','2','#': headings }
    nLines:INTEGER;

{ return true iff S is a string containing character C only }
FUNCTION IsChar(S:STRING; c:CHAR):BOOLEAN;
VAR i:INTEGER;
BEGIN
  FOR i:=1 TO Length(S) DO IF S[i]<>c THEN BEGIN
    IsChar:=FALSE;
    Exit;
  END;
  IsChar:=TRUE;
END;

{ error exit }
PROCEDURE Error(S:STRING);
BEGIN
  Writeln('ERROR: ',S);
  Halt(1);
END;

{ return true iff S triggers the line to be treated as preformatted text }
FUNCTION IsPreformatTrigger(S:STRING):BOOLEAN;
BEGIN
  IsPreformatTrigger :=
    (Copy(S,1,4)='VAR ') OR (Copy(S,1,4)='FOR ') OR (Pos(':=',S)<>0)
    OR (Copy(S,1,4)='--- ') OR (Copy(S,1,4)='<<< ') OR (Copy(S,1,3)='')
    OR (Copy(S,1,3)='IF ') OR (Copy(S,1,9)='FUNCTION ')
    OR (Copy(S,1,3)='/* ') OR (Copy(S,1,3)='** ')
    OR (Copy(S,1,3)='// ') OR (Copy(S,1,2)='+ ');
END;

{ Scanning pass: reads file and assigns line types }
PROCEDURE ReadFile(Name:STRING);
VAR F:TEXT;
    line:STRING;
    i:INTEGER;
BEGIN
  Assign(F, Name);
  Reset(F);
  IF IOResult<>0 THEN Error('Can''t open `'+Name+'''');

  nLines := 0;

  WHILE NOT EOF(F) DO BEGIN
    Readln(F, line);
    Inc(nLines);
    IF nLines>MAXLINES THEN Error('File too large: increase MAXLINES');
    IF line='' THEN Lines[nLines] := #0 ELSE
    IF IsChar(line, '=') THEN Lines[nLines] := '=' ELSE
    IF IsChar(line, '-') THEN Lines[nLines] := '-' ELSE
    IF line[1]='#' THEN Lines[nLines] := '#' ELSE
    BEGIN
      Strip(line, s_Both);
      IF (line<>'') AND (line[1]='') THEN Lines[nLines] := '' ELSE
      IF (Pos('  ',line)<>0) OR (IsPreformatTrigger(line)) THEN Lines[nLines] := ' ' ELSE
       Lines[nLines]:='A';
    END;
  END;

  Close(F);
END;

{ find next line after FROM which has type TYP, returns line number or -1 }
FUNCTION FindNext(typ:CHAR; from:INTEGER):INTEGER;
BEGIN
  WHILE (from < nLines) AND (from<>-1) DO BEGIN
    Inc(from);
    IF typ = lines[from] THEN BEGIN
      FindNext := from;
      Exit;
    END;
  END;
  FindNext := -1;
END;

{ find previous line before FROM which has type TYP, returns line number or
  -1 }
FUNCTION FindPrev(typ:CHAR; from:INTEGER):INTEGER;
BEGIN
  WHILE (from > 1) DO BEGIN
    Dec(from);
    IF typ = lines[from] THEN BEGIN
      FindPrev := from;
      Exit;
    END;
  END;
  FindPrev := -1;
END;

{ skips up to next non-blank line }
FUNCTION SkipPara(line:INTEGER):INTEGER;
BEGIN
  WHILE (line < nLines) AND (Lines[line]=#0) DO Inc(line);
  SkipPara:=line;
END;

{ Format pass: format the paragraph after LINE (skips blanks if needed),
  returns number of next line after paragraph, or NLINES+1 if none }
FUNCTION FormatPara(line:INTEGER):INTEGER;
VAR eop,i,j:INTEGER;
    c:CHAR;
BEGIN
  line := SkipPara(line);
  eop := FindNext(#0, line);
  IF eop < 0 THEN eop := nlines+1;

  IF Lines[line] = '#' THEN BEGIN
    FOR i:=line TO eop-1 DO Lines[i]:='#';
  END ELSE IF FindPrev('-', eop)>=line THEN BEGIN
    FOR i:=line TO eop-1 DO IF Lines[i]<>'-' THEN Lines[i]:='2';
  END ELSE BEGIN
   { possibly a text block, maybe followed by a bulleted list or text }
    WHILE (line < eop) AND (Lines[line]='A') DO Inc(line);
    IF (line < eop) THEN CASE Lines[line] OF
      '':BEGIN
        c:='b';
        FOR i:=line TO eop-1 DO IF Lines[i]='' THEN c:='b' ELSE BEGIN
          IF Lines[i]=' ' THEN c:='B';
          Lines[i]:=c;
        END;
      END;
      ' ':BEGIN
        FOR i:=line TO eop-1 DO Lines[i]:=' ';
      END;
    END;
  END;
  FormatPara:=eop;
END;

{ formats section whose heading starts after LINE, returns position
  of next blank line after this section }
FUNCTION FormatSection(line:INTEGER):INTEGER;
VAR eos:INTEGER;
BEGIN
  line := SkipPara(line);
  WHILE (line < nLines) AND ((Lines[line]='A') OR (Lines[line]=' ')) DO BEGIN
    Lines[line] := '1';
    Inc(line);
  END;
  IF (line >= nLines) OR (Lines[line]<>'=') THEN Error('Expected `==='' line');
  Inc(line);
 { find first text line }
  line := SkipPara(line);
  eos := FindPrev(#0, FindNext('=', line));
  IF eos < 0 THEN eos := nlines + 1;

  WHILE line < eos DO line := FormatPara(line);

  IF eos > nlines THEN FormatSection:=-1 ELSE FormatSection:=eos;
END;

{ Format pass: simplify some constructions }
PROCEDURE Postprocess;
VAR i:INTEGER;
BEGIN
  FOR i:=1 TO nLines-2 DO BEGIN
   { blank lines between bullet-list items are counted towards the text }
    IF ((Lines[i]='b') OR (Lines[i]='')) AND (Lines[i+1]=#0) AND (Lines[i+2]='') THEN
      Lines[i+1] := 'b';
    IF (Lines[i]='B') AND (Lines[i+1]=#0) AND (Lines[i+2]='') THEN
      Lines[i+1] := 'B';
   { blank lines between preformatted text sections do not split these }
   { sections }
    IF (Lines[i]=' ') AND (Lines[i+1]=#0) AND (Lines[i+2]=' ') THEN
      Lines[i+1] := ' ';
   { a single text line before a preformatted section is counted to }
   { preformatted }
    IF (Lines[i]=#0) AND (Lines[i+1]='A') AND (Lines[i+2]=' ') THEN
      Lines[i+1] := ' ';
  END;
END;

{ Format pass: format whole file }
PROCEDURE FormatFile;
VAR i,j:INTEGER;
    tlh:INTEGER;
    S:STRING;
BEGIN
  i := FindNext('=', 0);
  IF i<0 THEN Error('Can''t find first top-level heading');
  j := FindPrev(#0, i);
  IF j<0 THEN j:=1;
 { everything up to first top-level heading is centered }
  FOR i:=1 TO j-1 DO Lines[i] := 'c';
  WHILE j>0 DO j:=FormatSection(j);
  Postprocess;
END;

FUNCTION NameOfSection(ind:INTEGER):STRING;
VAR S:STRING[15];
BEGIN
  Str(ind, S);
  NameOfSection := 'text' + S + HTMLEXT;
END;

{ start new section (level-1 heading). Opens F as a file to write the
  section to [nul in first pass, html file in second] }
PROCEDURE NewSection(VAR F:TEXT);
BEGIN
  Inc(OutInd);
  OutHRef := NameOfSection(OutInd);
  IF Pass=1 THEN Assign(F,'nul') ELSE Assign(F, Path + OutHRef);
  Rewrite(F);
  IF IOResult<>0 THEN Error('Can''t create `'+OutHRef+'''');
  Write(OutInd:3,#8#8#8);
  IF OutInd > MaxOutInd THEN MaxOutInd := OutInd;
END;

{ Generate new anchor for a second-level heading }
PROCEDURE NewAnchor;
BEGIN
  Inc(OutAInd);
  Str(OutAInd, OutAnchor);
END;

{ true iff S is a word that can be a cross reference }
FUNCTION HRefableWord(S:STRING):BOOLEAN;
VAR i:INTEGER;
BEGIN
  HRefableWord := FALSE;
  IF S='' THEN Exit;
  FOR i:=1 TO Length(S) DO BEGIN
    IF (S[i]>='a') AND (S[i]<='z') AND (S[i]<>'x') THEN Exit;
  END;
  HRefableWord := TRUE;
END;

{ add cross references (and otherwise postprocess) word S }
FUNCTION HRefAdd(S:STRING):STRING;
VAR P:PHRef;
    i:INTEGER;
    c:CHAR;
BEGIN
  IF S<>'' THEN BEGIN
    IF (S[1]='_') AND (S[Length(S)]='_') THEN BEGIN
      FOR i:=1 TO Length(S) DO IF S[i]='_' THEN S[i]:=' ';
      Strip(S,s_Both);
      HRefAdd:='<em>'+S+'</em>';
      Exit;
    END;
    i := Pos('@', S);
    IF (i>1) AND (i<Length(S)-5) THEN BEGIN
      HRefAdd := '<a href="mailto:' + S + '">' + S + '</a>';
      Exit;
    END;
    IF (Copy(S, 1, 7)='http://') OR (Copy(S, 1, 6)='ftp://') THEN BEGIN
      HRefAdd := '<a href="' + S + '">' + S + '</a>';
      Exit;
    END;
    P:=HRefs;
    WHILE P<>NIL DO BEGIN
      IF P^.name = S THEN BEGIN
        HRefAdd := '<a href="' + P^.href + '">' + S + '</a>';
        Exit;
      END;
      P:=P^.Next;
    END;
  END;
  HRefAdd:=S;
END;

CONST Punctuation = ' -/,()[]{}!?+"''`';
      Punctuation2 = ' ,()[]{}!?+"''`';

FUNCTION FindEndOfWord(S:STRING):INTEGER;
VAR i:INTEGER;
    pun:STRING;
BEGIN
  i:=1;
  pun:=Punctuation;
  WHILE (i<Length(S)) AND (Pos(S[i+1], pun)=0) DO BEGIN
    Inc(i);
    IF S[i]=':' THEN pun:=Punctuation2;   { maybe it's going to be an URL }
  END;
  IF (i>1) AND (S[i]='.') THEN Dec(i);
  FindEndOfWord:=i;
END;

{ add references (and otherwise postprocess) string S }
FUNCTION HRefify(S:STRING):STRING;
VAR S1:STRING;
    i,j,k:INTEGER;
BEGIN
  IF Pass=1 THEN BEGIN
    HRefify:=S;
    Exit;
  END;
  S1:='';
  WHILE S<>'' DO BEGIN
    IF S[1]='<' THEN BEGIN
      i:=Pos('>', S);
      IF i=0 THEN BEGIN
        Writeln('WARNING: Internal error: `<'' not matched');
        i:=Length(S);
      END;
      S1:=S1+Copy(S,1,i);
      Delete(S,1,i);
    END ELSE IF S[1]='&' THEN BEGIN
      i:=Pos(';', S);
      IF i=0 THEN BEGIN
        Writeln('WARNING: Internal error: `&'' not matched');
        i:=Length(S);
      END;
      S1:=S1+Copy(S,1,i);
      Delete(S,1,i);
    END ELSE IF Pos(S[1], Punctuation)=0 THEN BEGIN
      i:=FindEndOfWord(S);
      S1:=S1+HRefAdd(Copy(S,1,i));
      Delete(S,1,i);
    END ELSE BEGIN
      S1:=S1+S[1];
      Delete(S,1,1);
    END;
  END;
  HRefify:=S1;
END;

{ add reference NAME to url HREF to the list of references }
PROCEDURE AddHRef(name, href:STRING);
VAR P:PHref;
BEGIN
  IF (Pass<>1) OR NOT HRefableWord(name) THEN Exit;
  P := Hrefs;
  WHILE (P<>NIL) DO IF P^.name=name THEN Exit ELSE P:=P^.Next;
  New(P);
  P^.name := name;
  P^.href := href;
  P^.next := Hrefs;
  Hrefs := P;
END;

{ add a level LEVEL heading NAME to the index }
PROCEDURE AddIndex(level:INTEGER; name,href:STRING);
VAR i:INTEGER;
BEGIN
  WHILE iNest < level DO BEGIN
    Writeln(Index, '<ul>':6+inest);
    Inc(inest);
  END;
  WHILE inest > level DO BEGIN
    Writeln(Index, '</ul>':6+inest);
    Dec(inest);
  END;
  IF level>0 THEN Writeln(Index,'<li>':6+inest,'<a href="'+href+'">'+ToHtml(name)+'</a>');
  IF level=2 THEN BEGIN
    Strip(name, s_Both);
    i:=Pos(' ',name);
    IF i>0 THEN Name[0] := Chr(i-1);
    AddHRef(name, href);
  END;
END;

{ skip lines in input file until one of type C reached, returns updated
  line number I }
FUNCTION SkipLines(i:INTEGER; VAR F:TEXT; c:CHAR):INTEGER;
BEGIN
  WHILE (i<=nLines) AND (Lines[i]=c) DO BEGIN
    Inc(i);
    Readln(F);
  END;
  SkipLines:=i;
END;

{ Output pass: output a paragraph read from input file F, written to
  output file T, line number is I. Returns line number after this
  paragraph }
FUNCTION OutputPara(VAR F,T:TEXT; i:INTEGER):INTEGER;
VAR S:STRING;
    flag:BOOLEAN;
BEGIN
  IF Lines[i]='2' THEN BEGIN
   { level-2 heading }
    NewAnchor;
    Write(T, '  <h2><a name="', OutAnchor, '">');
    flag:=FALSE;
    WHILE (i<=nLines) AND (Lines[i]='2') DO BEGIN
      IF flag THEN Write(T,'<br>'#13#10'   ');
      Readln(F, S);
      Write(T,ToHtml(S));
      AddIndex(2, S, OutHRef + '#' + OutAnchor);
      flag:=TRUE;
      Inc(i);
    END;
    Writeln(T, '</a></h2>');
    OutputPara := SkipLines(i, F, '-');
    Exit;
  END ELSE IF Lines[i]='#' THEN BEGIN
   { level-3 heading }
    Write(T, '  <h3>');
    flag:=FALSE;
    WHILE (i<=nLines) AND (Lines[i]='#') DO BEGIN
      IF flag THEN Write(T,{'<br>'}#13#10'   ');
      Readln(F, S);
      WHILE (S<>'') AND ((S[1]='#') OR (S[1]=' ')) DO Delete(S,1,1);
      Write(T,ToHtml(S));
      flag:=TRUE;
      Inc(i);
    END;
    Writeln(T, '</h3>');
    OutputPara := i;
    Exit;
  END ELSE BEGIN
   { paragraph optionally followed by preformatted text or bulleted list }
   { paragraph may be empty }
    flag:=FALSE;
    WHILE (i<=nLines) AND (Lines[i]='A') DO BEGIN
      Readln(F, S);
      Inc(i);
      IF NOT flag THEN Write(T,'  <p>') ELSE Write(T, #13#10'   ');
      Strip(S, s_Both);
      IF (Copy(S, 1, 2)='- ') AND flag THEN Write(T, '<br>');
      Write(T, HRefify(ToHtml(S)));
      flag := TRUE;
    END;
    IF flag THEN Writeln(T, '</p>');
    IF (i<=nLines) THEN BEGIN
     { bulleted list follows }
      IF Lines[i]='' THEN BEGIN
        Writeln(T,'  <ul>');
        flag:=FALSE;
        WHILE (i<=nLines) AND ((Lines[i]='') OR (Lines[i]='b') OR (Lines[i]='B')) DO BEGIN
          Readln(F, S);
          IF lines[i]='B' THEN BEGIN
           { preformatted list item }
            IF NOT flag THEN Writeln(T,'   <pre>');
            flag:=TRUE;
          END ELSE BEGIN
           { flow-text list item }
            IF Flag THEN Writeln(T,'   </pre>');
            flag:=FALSE;
            WHILE (S<>'') AND ((S[1]='') OR (S[1]=' ')) DO Delete(S,1,1);
            IF Lines[i]='' THEN Write(T,'   <li>') ELSE Write(T,'    ');
          END;
          Writeln(T, HRefify(ToHtml(S)));
          Inc(i);
        END;
        IF Flag THEN Writeln(T,'   </pre>');
        Writeln(T,'  </ul>');
      END ELSE IF Lines[i]<>#0 THEN BEGIN
       { preformatted section }
        IF Lines[i]<>' ' THEN Writeln('WARNING: expected ` '' type line (line ',i,')');
        Writeln(T,'  <pre>');
        WHILE (i<=nLines) AND (Lines[i]=' ') DO BEGIN
          Readln(F, S);
          Inc(i);
          IF (Copy(S,1,3)='---') THEN Writeln(T,'<i>',HRefify(ToHtml(S)),'</i>') ELSE
           Writeln(T, HRefify(ToHtml(S)));
        END;
        Writeln(T,'  </pre>');
      END;
    END;
    OutputPara := i;
  END;
END;

{ convert ALL-CAPS heading S into better readable Not-All-Caps form }
PROCEDURE Capitalize(VAR S:STRING);
VAR i:INTEGER;
    flag:BOOLEAN;
BEGIN
  flag:=TRUE;
  FOR i:=1 TO Length(S) DO IF (S[i]=' ') OR (S[i]='-') THEN Flag:=TRUE ELSE BEGIN
    IF (S[i]>='A') AND (S[i]<='Z') AND NOT flag THEN Inc(S[i],32);
    Flag:=FALSE;
  END;
END;

{ output section after line I, return line number after this section }
FUNCTION OutputSection(VAR F:TEXT; i:INTEGER):INTEGER;
VAR eos:INTEGER;
    T:TEXT;
    S:STRING;
BEGIN
  i := SkipLines(i,F,#0);
  IF (i>nLines) THEN BEGIN
    OutputSection:=nLines+1;
    Exit;
  END;
  IF Lines[i]<>'1' THEN Error('Internal error: expected H1 section');
  Readln(F,S);
  NewSection(T);
  Capitalize(S);
  StartHtml(T,ToHtml(S));
  AddIndex(1, S, OutHRef);
  Inc(i);
 { section heading }
  Write(T, '  <h1>',ToHtml(S));
  WHILE (i<=nLines) AND (Lines[i]='1') DO BEGIN
    Readln(F, S);
    Capitalize(S);
    Inc(i);
    AddIndex(1, S, OutHRef);
    Write(T, '<br>'#13#10'   ',ToHtml(S));
  END;
  Writeln(T,'</h1>');
 { now, emit rest of section }
  i := SkipLines(i, F, '=');
  i := SkipLines(i, F, #0);
  eos := FindNext('1', i);
  IF eos<0 THEN eos:=nLines+1;
  WHILE i<eos DO i:=SkipLines(OutputPara(F, T, i), F, #0);
  OutputSection:=i;

  IF OutInd > 1         THEN percent_vars['p'] := NameOfSection(OutInd-1) ELSE percent_vars['p'] := '';
  IF OutInd < MaxOutInd THEN percent_vars['n'] := NameOfSection(OutInd+1) ELSE percent_vars['n'] := '';

  Write(T, ReplaceVars(TextFooter));
  EndHtml(T);
  Close(T);
END;

{ Output pass: output file FN to html files }
PROCEDURE Output(Fn:STRING);
VAR F:TEXT;
    i:INTEGER;
    S:STRING;
BEGIN
  Assign(F, Fn);
  Reset(F);
  IF IOResult<>0 THEN Error('Can''t open `'+Fn+'''');
  i:=1;
  IF (nLines>=1) AND (Lines[1]='c') THEN BEGIN
    Write(Index, '  <center>');
    WHILE (i <= nLines) AND (Lines[i]='c') DO BEGIN
      IF i>1 THEN Write(Index, '<br>'#13#10'   ');
      Readln(F, S);
      Strip(S, s_Both);
      Write(Index, ToHtml(S));
      Inc(i);
    END;
    Writeln(Index,'</center>');
  END;
  Writeln(Index,'  <h1>Table of Contents</h1>');
  iNest := 0;
  WHILE (i<=nLines) DO i:=OutputSection(F, i);
  Close(F);
END;

PROCEDURE LoadString(VAR F:TEXT; S:STRING; VAR Target:STRING);
VAR i:INTEGER;
BEGIN
  Target := '';
  i := Pos('{', S);
  Delete(S, 1, i);
  IF S<>'' THEN Target := S+#13#10;
  WHILE (Pos('}', Target)=0) AND (Length(Target) < 255) DO BEGIN
    IF EOF(F) THEN Error('Unmatched `{'' in stylesheet.');
    Readln(F, S);
    Target := Target + S + #13#10;
  END;
  i := Pos('}', Target);
  IF i>0 THEN Target[0] := Chr(i-1);
END;

PROCEDURE LoadInfos(S:STRING);
VAR F:TEXT;
    S1:STRING;
    i:INTEGER;
BEGIN
  Assign(F, S);
  Reset(F);
  IF IOResult<>0 THEN Error('Can''t open stylesheet ' + S);
  WHILE NOT EOF(F) DO BEGIN
    Readln(F, S);
    IF (S<>'') AND (S[1]<>';') THEN BEGIN
      i := Pos('->', S);
      IF i<>0 THEN BEGIN
        S1 := Copy(S, i+2, 255);
        Strip(S1, s_Both);
        S[0] := Chr(i-1);
        Strip(S, s_Both);
        AddHref(S, S1);
      END ELSE
      IF Copy(S, 1, 6) = 'head {' THEN BEGIN
        LoadString(F, S, StartTemplate);
      END ELSE
      IF Copy(S, 1, 6) = 'tail {' THEN BEGIN
        LoadString(F, S, EndTemplate);
      END ELSE
      IF Copy(S, 1, 13) = 'text_footer {' THEN BEGIN
        LoadString(F, S, TextFooter);
      END ELSE
      IF Copy(S, 1, 14) = 'index_footer {' THEN BEGIN
        LoadString(F, S, IndexFooter);
      END ELSE
       Error('Unrecognized stylesheet line ' + S);
    END;
  END;
  Close(F);
END;

BEGIN
  IF (ParamCount<>2) AND (ParamCount<>3) THEN BEGIN
    Writeln('usage: txt2html inputfile outputdirectory [stylesheet]');
    Halt(1);
  END;
  Path:=ParamStr(2);
  IF (Path<>'') THEN CASE (Path[Length(Path)]) OF
    '\','/',':':;
  ELSE Path:=Path+'\';
  END;

  Pass := 1;    { fr AddHref }
  HRefs := NIL;
  IF ParamStr(3)<>'' THEN LoadInfos(ParamStr(3));
  percent_vars['s'] := ParamStr(1);

 { Scanner pass: reads the source file for first time, assigns each line a }
 { `type'. This gives line types ` A#=-'#0 (pre, text, bullet, underlinings }
 { and blank lines) }
  Write('Scanning file...');
  ReadFile(ParamStr(1));
  Writeln(' done.');

 { Format pass: finds paragraphs, headings, etc. This adds types `cbB12' }
 { (centered heading, bullet-list text/pre, level-1/2 heading) and simplifies }
 { others }
  Write('Formatting...');
  FormatFile;
  Writeln(' done.');

 { First output pass: read source file, write index, find cross reference }
 { targets; HTML output directed to nul }
  OutInd:=0;
  MaxOutInd:=0;
  OutAInd:=0;
  Assign(Index, Path + 'index' + HTMLEXT);
  Rewrite(Index);
  IF IOResult<>0 THEN Error('Can''t create ' + path + 'index' + HTMLEXT);
  StartHtml(Index, 'Table of Contents');

  Write('Output (pass 1)...');
  Pass := 1;
  Output(ParamStr(1));
  AddIndex(0,'','');
  Write(Index, ReplaceVars(IndexFooter));
  EndHtml(Index);
  Close(Index);
  Writeln(' done.');

 { Second output pass: read source file, write text files, write formatted/ }
 { xref'ed text, index output directed to NUL }
  Assign(Index, 'nul');
  Rewrite(Index);
  Write('Output (pass 2)...');
  Pass := 2;
  OutInd:=0;
  OutAInd:=0;
  Output(ParamStr(1));
  AddIndex(0,'','');
  Close(Index);
  Writeln(' done.');
END.
